@@ -574,17 +574,30 @@ class SyncObj(object):
                     self.__onBecomeLeader()
 
         if self.__raftState == _RAFT_STATE.LEADER:
-            while self.__raftCommitIndex < self.__getCurrentLogIndex():
-                nextCommitIndex = self.__raftCommitIndex + 1
+
+            commitIdx = self.__raftCommitIndex
+            nextCommitIdx = self.__raftCommitIndex
+
+            while commitIdx < self.__getCurrentLogIndex():
+                commitIdx += 1
                 count = 1
                 for node in self.__otherNodes:
-                    if self.__raftMatchIndex[node] >= nextCommitIndex:
+                    if self.__raftMatchIndex[node] >= commitIdx:
                         count += 1
-                if count > (len(self.__otherNodes) + 1) / 2:
-                    self.__raftCommitIndex = nextCommitIndex
-                    self.__raftLog.setRaftCommitIndex(self.__raftCommitIndex)
-                else:
+                if count <= (len(self.__otherNodes) + 1) / 2:
                     break
+                entries = self.__getEntries(commitIdx, 1)
+                if not entries:
+                    continue
+                commitTerm = entries[0][2]
+                if commitTerm != self.__raftCurrentTerm:
+                    continue
+                nextCommitIdx = commitIdx
+
+            if self.__raftCommitIndex != nextCommitIdx:
+                self.__raftCommitIndex = nextCommitIdx
+                self.__raftLog.setRaftCommitIndex(self.__raftCommitIndex)
+
             self.__leaderCommitIndex = self.__raftCommitIndex
             deadline = monotonicTime() - self.__conf.leaderFallbackTimeout
             count = 1
@@ -908,7 +921,7 @@ class SyncObj(object):
 
                 nextNodeIdx = prevLogIdx + 1
                 if newEntries:
-                    nextNodeIdx = newEntries[-1][1]
+                    nextNodeIdx = newEntries[-1][1] + 1
 
                 self.__sendNextNodeIdx(node, nextNodeIdx=nextNodeIdx, success=True)
 
@@ -918,7 +931,9 @@ class SyncObj(object):
                     self.__loadDumpFile(clearJournal=True)
                     self.__sendNextNodeIdx(node, success=True)
 
-            self.__raftCommitIndex = min(leaderCommitIndex, self.__getCurrentLogIndex())
+            if leaderCommitIndex > self.__raftCommitIndex:
+                self.__raftCommitIndex = min(leaderCommitIndex, self.__getCurrentLogIndex())
+
             self.__raftLog.setRaftCommitIndex(self.__raftCommitIndex)
 
         if message['type'] == 'apply_command':
@@ -957,7 +972,9 @@ class SyncObj(object):
                 if reset:
                     self.__raftNextIndex[node] = nextNodeIdx
                 if success:
-                    self.__raftMatchIndex[node] = currentNodeIdx
+                    if self.__raftMatchIndex[node] < currentNodeIdx:
+                        self.__raftMatchIndex[node] = currentNodeIdx
+                        self.__raftNextIndex[node] = nextNodeIdx
                 self.__lastResponseTime[node] = monotonicTime()
 
     def __callErrCallback(self, err, callback):
